@@ -1,7 +1,7 @@
 /* dns.h - Declarations for dns handling and generic dns functions
 
    Copyright (C) 2000, 2001 Thomas Moestl
-   Copyright (C) 2002, 2003, 2004, 2005, 2009 Paul A. Rombouts
+   Copyright (C) 2002, 2003, 2004, 2005, 2009, 2011 Paul A. Rombouts
 
   This file is part of the pdnsd package.
 
@@ -20,7 +20,6 @@
   <http://www.gnu.org/licenses/>.
 */
 
-/* $Id: dns.h,v 1.15 2003/04/06 23:02:46 tmm Exp $ */
 
 #ifndef DNS_H
 #define DNS_H
@@ -103,16 +102,17 @@
 #define RC_NAMEERR  3
 #define RC_NOTSUPP  4
 #define RC_REFUSED  5
+#define RC_BADVERS 16
 
 /*
  * special internal retvals
  */
-#define RC_NOTCACHED  250
-#define RC_CACHED     251
-#define RC_STALE      252
-#define RC_TCPREFUSED 253
-#define RC_TRUNC      254
-#define RC_FATALERR   255
+#define RC_NOTCACHED  0xfffa
+#define RC_CACHED     0xfffb
+#define RC_STALE      0xfffc
+#define RC_TCPREFUSED 0xfffd
+#define RC_TRUNC      0xfffe
+#define RC_FATALERR   0xffff
 
 /* query/response */
 #define QR_QUERY    0
@@ -141,6 +141,8 @@ typedef struct {
 #define sizeof_rr_hdr_t 10
 #endif
 
+#define sizeof_opt_pseudo_rr  (1+sizeof_rr_hdr_t)
+
 #if 0
 typedef struct {
 	/* The server name and maintainer mailbox are the first two fields. It has variable length, */
@@ -195,6 +197,30 @@ typedef struct {
 } __attribute__((packed)) dns_hdr_t;
 
 
+/* A structure that can also be used for DNS messages over TCP. */
+typedef struct {
+#ifndef NO_TCP_QUERIES
+	uint16_t len;
+#endif
+	dns_hdr_t hdr;
+} __attribute__((packed)) dns_msg_t;
+
+#ifdef NO_TCP_QUERIES
+# define dnsmsghdroffset 0
+#else
+# define dnsmsghdroffset 2
+#endif
+
+
+/* Structure for storing EDNS (Extension mechanisms for DNS) information. */
+typedef struct {
+	unsigned short udpsize;
+	unsigned short rcode;
+	unsigned short version;
+	unsigned char  do_flg;
+} edns_info_t;
+
+
 /* Macros to retrieve or store integer data that is not necessarily aligned.
    Also takes care of network to host byte order.
    The pointer cp is advanced and should be of type void* or char*.
@@ -241,6 +267,9 @@ typedef struct {
 } while (0)
 
 
+/* Size (number of bytes) of buffers used to hold domain names. */
+#define DNSNAMEBUFSIZE 256
+
 /* Recursion depth. */
 #define MAX_HOPS 20
 
@@ -248,30 +277,33 @@ typedef struct {
  * Types for compression buffers.
  */
 typedef struct {
-	int           index;
+	unsigned int  index;
 	unsigned char s[0];
 } compel_t;
 
 
 
-int decompress_name(unsigned char *msg, long msgsz, unsigned char **src, long *sz, unsigned char *tgt, int *len);
+int decompress_name(unsigned char *msg, size_t msgsz, unsigned char **src, size_t *sz, unsigned char *tgt, unsigned int *len);
 /* int domain_name_match(const unsigned char *ms, const unsigned char *md, int *os, int *od); */
-int domain_match(const unsigned char *ms, const unsigned char *md, int *os, int *od);
-int compress_name(unsigned char *in, unsigned char *out, int offs, dlist *cb);
+unsigned int domain_match(const unsigned char *ms, const unsigned char *md, unsigned int *os, unsigned int *od);
+unsigned int compress_name(unsigned char *in, unsigned char *out, unsigned int offs, dlist *cb);
 int a2ptrstr(pdnsd_ca *a, int tp, unsigned char *buf);
 int read_hosts(const char *fn, unsigned char *rns, time_t ttl, unsigned flags, int aliases, char **errstr);
 
+const char *getrrtpname(int tp);
 #if DEBUG>0 
 const char *get_cname(int id);
 const char *get_tname(int id);
 const char *get_ename(int id);
+#define DNSFLAGSMAXSTRSIZE (7*3+1)
+char *dnsflags2str(dns_hdr_t *hdr, char *buf);
 #endif
 
 #if DEBUG>=9
-void debug_dump_dns_msg(pdnsd_a *a, void *data, size_t len);
-#define DEBUG_DUMP_DNS_MSG(a,d,l) {if(debug_p && global.verbosity>=9) debug_dump_dns_msg(a,d,l);}
+void debug_dump_dns_msg(void *data, size_t len);
+#define DEBUG_DUMP_DNS_MSG(d,l) {if(debug_p && global.verbosity>=9) debug_dump_dns_msg(d,l);}
 #else
-#define DEBUG_DUMP_DNS_MSG(a,d,l)
+#define DEBUG_DUMP_DNS_MSG(d,l)
 #endif
 
 #endif
